/*(c) Copyright 2015 Pivotal Software, Inc. All Rights Reserved.*/
'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _objectWithoutProperties = require('babel-runtime/helpers/object-without-properties')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var classnames = require('classnames');
var React = require('react');

var types = React.PropTypes;

var DOWN_KEY = 40;
var ENTER_KEY = 13;
var ESC_KEY = 27;
var TAB_KEY = 9;
var UP_KEY = 38;

function waitForRelatedTarget() {
  return new _Promise(function (resolve) {
    setImmediate(function () {
      resolve(document.activeElement);
    });
  });
}

function withRelatedTarget(callback) {
  return function callee$1$0(e) {
    return _regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          if (e.relatedTarget) {
            context$2$0.next = 5;
            break;
          }

          e = _extends({}, e);
          context$2$0.next = 4;
          return _regeneratorRuntime.awrap(waitForRelatedTarget.call(this));

        case 4:
          e.relatedTarget = context$2$0.sent;

        case 5:
          return context$2$0.abrupt('return', callback.call(this, e));

        case 6:
        case 'end':
          return context$2$0.stop();
      }
    }, null, this);
  };
}

var AutocompleteInput = React.createClass({
  displayName: 'AutocompleteInput',

  propTypes: {
    $autocomplete: types.object,
    autoFocus: types.bool,
    children: function children(props, name) {
      if (props[name] && props[name].length) return new Error('AutocompleteInput can only wrap one element');
    },
    disabled: types.bool,
    hideList: types.func,
    onClick: types.func,
    onFocus: types.func,
    onPick: types.func,
    onPicking: types.func,
    onSearch: types.func,
    scrollIntoView: types.func
  },

  statics: {
    DOWN_KEY: DOWN_KEY,
    ENTER_KEY: ENTER_KEY,
    ESC_KEY: ESC_KEY,
    TAB_KEY: TAB_KEY,
    UP_KEY: UP_KEY
  },

  getDefaultProps: function getDefaultProps() {
    return { autoFocus: null };
  },

  blur: withRelatedTarget(function (_ref) {
    var relatedTarget = _ref.relatedTarget;

    if (relatedTarget && relatedTarget.classList && relatedTarget.classList.contains('autocomplete-item')) return;
    this.props.hideList();
  }),

  change: function change(e) {
    var _this = this;

    var value = e.currentTarget.value;

    this.props.onSearch(value, function (suggestedValues) {
      _this.props.$autocomplete.merge({ hidden: false, highlightedSuggestion: 0, value: value, suggestedValues: suggestedValues }).flush();
    });
  },

  keyDown: function keyDown(e) {
    var _keyCodes,
        _this2 = this;

    var keyCode = e.keyCode;

    var _props$$autocomplete$get = this.props.$autocomplete.get();

    var highlightedSuggestion = _props$$autocomplete$get.highlightedSuggestion;
    var suggestedValues = _props$$autocomplete$get.suggestedValues;
    var _props$onPicking = this.props.onPicking;
    var onPicking = _props$onPicking === undefined ? function () {
      return suggestedValues;
    } : _props$onPicking;

    var pickItem = function pickItem() {
      var selectableSuggestions = onPicking(suggestedValues);
      e && keyCode === ENTER_KEY && e.preventDefault();
      _this2.props.$autocomplete.merge({ highlightedSuggestion: -1, hidden: true }).flush();
      _this2.props.onPick(selectableSuggestions[highlightedSuggestion] || { value: _this2.props.$autocomplete.get('value') });
    };

    var keyCodes = (_keyCodes = {}, _keyCodes[DOWN_KEY] = function () {
      var selectableSuggestions = onPicking(suggestedValues);
      _this2.props.$autocomplete.merge({ hidden: false, highlightedSuggestion: Math.min(highlightedSuggestion + 1, selectableSuggestions.length - 1) });
      _this2.props.scrollIntoView();
    }, _keyCodes[UP_KEY] = function () {
      _this2.props.$autocomplete.merge({ highlightedSuggestion: Math.max(highlightedSuggestion - 1, -1) });
      _this2.props.scrollIntoView();
    }, _keyCodes[TAB_KEY] = pickItem, _keyCodes[ENTER_KEY] = pickItem, _keyCodes[ESC_KEY] = function () {
      _this2.props.$autocomplete.merge({ highlightedSuggestion: -1, hidden: true });
    }, _keyCodes.noop = function noop() {}, _keyCodes);

    keyCodes[keyCode in keyCodes ? keyCode : 'noop']();
  },

  renderDefault: function renderDefault(props) {
    return React.createElement('input', _extends({}, props, { className: classnames('autocomplete-input', 'form-control', props.className), type: 'search', value: props.value, 'aria-label': props.placeholder }));
  },

  render: function render() {
    var _props = this.props;
    var autoFocus = _props.autoFocus;
    var children = _props.children;
    var $autocomplete = _props.$autocomplete;

    var props = _objectWithoutProperties(_props, ['autoFocus', 'children', '$autocomplete']);

    if (!$autocomplete) return null;

    var _$autocomplete$get = $autocomplete.get();

    var value = _$autocomplete$get.value;

    var otherProps = { autoFocus: autoFocus, value: value, onBlur: this.blur, onChange: this.change, onKeyDown: this.keyDown };
    props = _extends({}, props, otherProps);
    if (!children) return this.renderDefault(props);
    children = React.Children.map(children, function (e) {
      return React.cloneElement(e, props);
    });
    return React.createElement(
      'div',
      null,
      children
    );
  }
});

module.exports = AutocompleteInput;